import twitterQuery as tQ
import numpy as np
import matplotlib.pyplot as plt
from cPickle import dump, load


#--Formulary lists the "drugs" to search for on Twitter

#formulary = ['cocaine','marijuana','bath salts','heroin','lsd','valium','ativan','ecstasy','ketamine']
formulary = ['coffee','beer']


graph_stats = []
for count,drug in enumerate(formulary):
	print 'Querying Twitter for ',drug,' and writing the results to an XLS file.'
	query = tQ.twitterQuery(drug)
	degree,path_length = query.make_graph()
	print 'Finding the Retweet Graph. Saving it and a rank-sorted histogram of the degrees of its nodes.'
	graph_stats.append((degree,path_length))

with open('graph_stats.p','wb') as file:
	dump([formulary,graph_stats],file)

fig = plt.figure()
ax1 = fig.add_subplot(211)
ax1.boxplot([data[0] for data in graph_stats])
ax2 = fig.add_subplot(212)
ax2.boxplot([data[1] for data in graph_stats]) 
plt.title('Graph theoretic differences among commonly abused drugs.')
ax2.xaxis.set_ticks(range(1,len(formulary)+1),formulary)

plt.savefig('summary.png',dpi=300)
plt.show()


import numpy as np
import matplotlib.pyplot as plt
from numpy import loadtxt

# creation of the data
name_list = ['day1', 'day2', 'day3', 'day4']
filename = "nduh_ape.summary"
data = loadtxt(filename, delimiter=':', dtype = {'names': ('drugs','count'),'formats':('S10','i4')})
print data
colors_list = ['0.5', 'r', 'b', 'g'] #optional

def customize_barh(data, width_bar=1, width_space=0.5, colors=None):
    
    n = len(data) #Number of different categories
    nobs = sum([x[1] for x in data])
    print nobs
    
    #some calculation to determine the position of Y ticks labels
    total_space = (n*width_bar)+(n-1)*width_space
    ind_space = n*width_bar
    step = ind_space/2.
    pos = np.arange(step, total_space+width_space, ind_space+width_space)

    # create the figure and the axes to plot the data 
    fig = plt.figure(figsize=(8,6))
    ax = fig.add_axes([0.15, 0.15, 0.65, 0.7])

    # remove top and right spines and turn ticks off if no spine
    ax.spines['right'].set_color('none')
    ax.spines['top'].set_color('none')
    ax.xaxis.set_ticks_position('bottom')
    ax.yaxis.set_ticks_position('right')    # ticks position on the right
    # postition of tick out
    ax.tick_params(axis='both', direction='out', width=3, length=6,
                   labelsize=24, pad=8)
    ax.spines['left'].set_linewidth(3)
    ax.spines['bottom'].set_linewidth(3)

    # plot the data
    for i,(drug,count) in enumerate(data):
        if colors == None:
            ax.barh(pos-step+i*width_bar, 100*round(count/float(nobs),2), width_bar, #facecolor='0.4',
                    edgecolor='k', linewidth=3)
        else:
            ax.barh(pos-step+i*width_bar, 100*round(count/float(nobs),2), width_bar, facecolor=colors[i],
                    edgecolor='k', linewidth=3)


    ax.set_yticks(pos)
    # you may want to use the list of name as argument of the function to be more
    # flexible (if you have to add a people)
    print [x[0] for x in data]
    ax.set_yticklabels(tuple([x[0] for x in data]))         
    ax.set_ylim((-width_space, total_space+width_space))
    ax.set_xlabel('Percentage of Tweets', size=26, labelpad=10)

customize_barh(data)
plt.savefig('perf.png')
plt.show()


#Calculate FleshKincaid from Tweets

from nltk import * #Such a hack
from simplejson import load
from readabilitytests import ReadabilityTool

#Attribution: ReadabilityTests comes from NTLKs Google Project page.

formulary = ['cocaine','marijuana','bath salts','heroin','opium','lsd','valium','ativan','ecstasy','ketamine']

def clean(tweet):
    verboten = ['@','RT','http://','#']
    handles = set(filter(lambda x: 1 in [symbol in x for symbol in verboten], tweet.split(' ')))
    return ' '.join([word for word in tweet.split(' ') if word not in handles])

def getFK(text=text):
	grader=ReadabilityTool(text)
	return grader.FleschKincaidGradeLevel()

grade_level=[]
for drug in formulary:
	filename = "results_"+drug.replace(' ','_')+".txt"
	with open(filename,'r') as f:
		data = load(f)		
		grade_level.append(getFK(data[0]['results'][0]['text']))
		
print grade_level




import nltk
import simplejson
from os import getcwd
from nltk.corpus import PlaintextCorpusReader
from nltk.text import Text
import xlwt as xlwt
from sys import argv
#-------------------------------------------------------------------------------------
#############################HELPER FUNCTIONS#########################################
#-------------------------------------------------------------------------------------
print sys.argv
def tweettext_aslist(line):
    return  [result['text'] for result in eval(line)['results']]

def clean(tweet):
    #Returns true if any of Nick's criteria are met
    #A tweet is invalid if the only time marijuana is mentioned is in somebody's handle
    handles = set(filter(lambda x: '@' in x or 'RT' in x or 'http://' in x , tweet.split(' ')))
    without_handles = [word for word in tweet.split(' ') if word not in handles]
    print ' '.join(without_handles)
    return ' '.join(without_handles)
#-------------------------------------------------------------------------------------
#############################LOADING TWEETS & SAVING XLS##############################
#-------------------------------------------------------------------------------------
#Load the txt file that contains the twitter results in JSON format
filename = 'results_marijuana.txt'
listing = open(filename,'r')
dictionary = {}

for linenumber,line in enumerate(listing.readlines()):
    dictionary[str(linenumber)] = tweettext_aslist(line)

corpus = [tweets for tweets in dictionary.itervalues()]
corpus = [tweet for tweetlist in corpus for tweet in tweetlist]
corpus_unique = set(corpus)
#------------------------------------------------------------------------------------
#Make XLS Workbook with a sheet for everyone
xls_wbk = xlwt.Workbook()
everyone = {'Nick','Alex','Dan','Jen','Mike'}
column = 1
for person in everyone:
	sheet = xls_wbk.add_sheet(person)
	for row,tweet in enumerate(map(clean,set(corpus))):
		sheet.write(row,column,tweet)
xls_wbk.save('marijuana.xls')
#--------------------------------------------------------------------------------------
''' 
	Save just the tweet portion of the Twitter query to a new txt file. Otherwise, NLTK
	won't be able to deal with it. 
	'''
savename ='m_bathsalts.txt'
with open(savename, 'w') as f:
	simplejson.dump(corpus,f)

corpus_legit = PlaintextCorpusReader(getcwd(),savename)
sanitized = [word for word in corpus_legit.words('m_bathsalts.txt') if word.isalnum()]
#-------------------------------------------------------------------------------------
#####################################ANALYSIS#########################################
#-------------------------------------------------------------------------------------
#Lexical Diveristy
print 'Lexical Diversity is:', len(map(clean,corpus))/float(len(map(clean(set(corpus))))
#-------------------------------------------------------------------------------------
#Frequency of most common words
#-------------------------------------------------------------------------------------
for_semantic_analysis = Text(corpus_legit.words())
for_semantic_analysis.similar('bath salts')
stemmed = map(nltk.PorterStemmer().stem,corpus_legit.words())



from numpy import *
from nltk import NaiveBayesClassifier, classify, FreqDist
from xlrd import open_workbook

#Load the coded data
def retrieve_tweets(drug_name):
	wb = open_workbook('coding_master.xls')
	master_sheet = wb.sheet_by_index(0)
	return zip(master_sheet.col_values(2),master_sheet.col_values(0))

def clean(tweet):
	return (filter(lambda x: len(x)>2,tweet[0].split()),tweet[1])
	
def split_by_category(categorized_tweets):
	answer = {}
	for text,rating in categorized_tweets:
		if int(rating) not in answer:
			answer[int(rating)] = text
		else:
			answer[int(rating)].extend(text)
	return answer
formulary = ['cocaine'] 
tweets = [map(clean,retrieve_tweets(drug)) for drug in formulary]

stratified_by_rating = split_by_category(tweets[0])
freq_dists = [FreqDist(text) for text in stratified_by_rating.itervalues()]
word_lists = map(lambda x:x.keys(),freq_dists)

all_together_word_list = [word for word_list in word_lists for word in word_list]

def extract_features(document):
	return  {'contains(%s)'% word: (word in set(document)) for word in all_together_word_list} 
training_set = classify.util.apply_features(extract_features,tweets[0])

classifier = NaiveBayesClassifier.train(training_set)
#print classifier.show_most_informative_features()
tweet = 'I am going to school.'
print classifier.classify(extract_features(tweet.split()))


#!/usr/bin/env python
# Using twitter instead of python-twtter wrapper
import twitter
import networkx as nx
import re
import codecs
import csv
import xlwt
from httplib import BadStatusLine
class twitterQuery(object):
    def __init__ (self, drug_name):
        self.drug_name = drug_name
        try: #Look for a list of synonyms. If no list is found search Twitter jut using the drug name
            self.synonym_filename = self.drug_name + '_synonyms.csv'
            self.synonyms = [line for line in csv.reader(open(self.synonym_filename,'rt'))]
            self.synonyms = [synonym for line in self.synonyms for synonym in line]
        except IOError as e:
             self.synonyms = []
             self.synonyms.append(self.drug_name)
             # Calling list(self.drug_name) splits the string into a list of characters 
        #Assume that the synonyms are stored in a CSV file
        #Query Twitter with those synonyms-------------------------------------
        self.search = twitter.Twitter(domain="search.twitter.com")
        self.results = []
        self.page_count = 10
        self.radius = 10    
        self.locations = {'NYC':'40.665572,-73.923557'}
        self.rpp = 100
        self.query = ' OR'.join(self.synonyms)
        
        self.dial_in_tries = 4 #Sometimes Twitter's API flips a BadStatusLine error
        for attempt in range(self.dial_in_tries):
            try:
                for page in range(1,self.page_count+1): # Python countings from 0. Twitter from one.
                    self.results.append(self.search.search(q=self.query, 
                    rpp=100,page=page, geocode=self.locations['NYC']+','+str(self.radius)+'mi'))
                break
            except BadStatusLine,e:
                continue
	#self.save(format='XLS')
	self.save(format='JSON')
	
    def clean(self,tweet):
        verboten = ['@','RT','http://']
        handles = set(filter(lambda x: 1 in [symbol in x for symbol in verboten], tweet.split(' ')))
        return ' '.join([word for word in tweet.split(' ') if word not in handles])
   
    def save(self, format='XLS'):
        if format == 'XLS': #Saving to an XLS file for human rating
            import xlwt as xlwt
            corpus = [tweet['text'] for page in self.results for tweet in page['results']]
            #Make XLS Workbook with a sheet for everyone
            self.xls_wbk = xlwt.Workbook()
            self.everyone = {'Nick','Alex','Dan','Jen','Mike'}
            self.column = 1
            for person in self.everyone:
                sheet = self.xls_wbk.add_sheet(person)
                for row,tweet in enumerate(map(self.clean,set(corpus))):
                    sheet.write(row,self.column,tweet)
            self.xls_wbk.save(self.drug_name.replace(' ','_')+'_for_rating.xls')
        elif format == 'JSON': #Dumping tweets, for say graph theory analysis
            from simplejson import dump
            self.record_filename = 'results_'+self.drug_name.replace(' ','_')+'.txt'
            with open(self.record_filename,'a') as record:
                dump(self.results,record)
    
    def get_rt_sources(self,tweet):
        re_patterns = re.compile(r'(RT|via)((?:\b\W*@\w+)+)')
        return [source.strip() for entry in re_patterns.findall(tweet)
                    for source in entry if source not in ("RT","via")]
                    
    def make_graph(self,save_graph=True):
        graph = nx.DiGraph()
        all_tweets = [tweet for page in self.results for tweet in page['results']]
        for tweet in all_tweets:
            rt_sources = self.get_rt_sources(tweet["text"])
            if not rt_sources: continue 
            for rt_source in rt_sources:
                graph.add_edge(rt_source, tweet["from_user"], {"tweet_id": tweet["id"]})
        #--Calculate graph summary statistics
        if nx.is_connected(graph.to_undirected()):
            diameter  = nx.diameter(graph.to_undirected())         
            average_shortest_path = nx.average_shortest_path_length(graph.to_undirected())
            print 'Diameter: ', diameter
            print 'Average Shortest Path: ',average_shortest_path
        else:
             print "Graph is not connected so calculating the diameter and average shortest path length on all connected components."
             diameter = []
             average_shortest_path = []
             for subgraph in nx.connected_component_subgraphs(graph.to_undirected()):
                 diameter.append(nx.diameter(subgraph))
                 average_shortest_path.append(nx.average_shortest_path_length(subgraph))
             from numpy import median
             from scipy.stats import scoreatpercentile
             print 'Diameter: ',median(diameter),u'\xB1',str(scoreatpercentile(diameter,75)-scoreatpercentile(diameter,25))
             print 'Average Path Length :',median(average_shortest_path),u'\xB1',str(scoreatpercentile(average_shortest_path,75)-scoreatpercentile(average_shortest_path,25))
        degree_sequence=sorted(nx.degree(graph).values(),reverse=True) # degree sequence
           
        import matplotlib.pyplot as plt
        plt.loglog(degree_sequence,'b-',marker='o')
        plt.title("Distribution of Degrees for %s tweets" %(self.drug_name), fontsize=20)
        plt.ylabel("Degree", fontsize=20)
        plt.xlabel("Rank", fontsize=20)
        
        # draw graph in inset
        ax = plt.axes([0.35,0.25,0.55,0.55])
        plt.axis('off')
        nx.draw(graph, ax=ax, alpha=0.8, with_labels=False)
        
        plt.savefig("degree_distribution_%s.png"%(self.drug_name.replace(' ','_')), dpi=300)
        plt.close()
        if save_graph:
            output_file = self.drug_name.replace(' ','_') + '.dot'
            try:
                nx.drawing.write_dot(graph,output_file)
                print 'Graph saved as ',output_file
            except (ImportError, UnicodeEncodeError) as e:
                dot = ['"%s" -> "%s" [tweetid=%s]' % (node1,node2,graph[node1][node2]['tweet_id']) 
                        for node1,node2, in graph.edges()]
                with codecs.open(output_file,'w', encoding='utf-8') as f:
                    f.write('strict digraph G{\n%s\n}' % (';\n'.join(dot),))
                    print 'Saved ',output_file,' by brute force'
        return diameter, average_shortest_path
                    


from lexdiv import ld
from pprint import pprint
filenames = ['ketamine','cocaine','bathsalts','ativan','MDMA','heroin','marijuana','lsd']
suffix = '1A'
diversity = [ld('.'.join([file,suffix])) for file in sorted(filenames)]

results = {filename:diverse for (filename,diverse) in zip(sorted(filenames),diversity)}
print 'Drug :::::::::: Lexical Diversity'
pprint(results.items())


# -*- coding: utf-8 -*-
# Sets the encoding to utf-8 to avoid problems with æøå
import urllib
import htmlentitydefs
from sgmllib import *
import re

class URLextracter(SGMLParser):
   
    style = False
    script = False
    hyperlink = False
   
    linklist = []
   
    def __init__(self, url='http://python.org'):
        self.reset()
        try:
            self.sock = urllib.urlopen(url)
            self.feed(self.sock.read())
            self.sock.close()
            self.close()
        except IOError:
            print "Could not connect, or the markup has (too) bad structure"
            raise SGMLParseError
       
    def start_style(self,attr):
        self.style = True
   
    def end_style(self):
        self.style = False
       
    def start_script(self,attr):
        self.script = True
   
    def do_script(self,attr):
        pass
   
    def end_script(self):
        self.script = False
       
    def start_a(self,attr):
        self.hyperlink = True
        href = [v for k, v in attr if k=='href']
        if href:
            self.linklist.extend(href)
   
    def end_a(self):
        self.hyperlink = False
   
    def reset(self):
        self.pieces = []
        self.linklist = []
        SGMLParser.reset(self)
       
    def handle_data(self, text):
        #tags_to_skip = [self.style, self.script, self.hyperlink]
        tags_to_skip = [self.style, self.script]
        if True in tags_to_skip:
            pass
        else:
            self.pieces.append(text)
   
    def handle_declaration(self,decl):
        pass
   
    def report_unbalanced(self,tag):
        pass
   
    def unknown_starttag(self, tag, attrs):
        pass

    def unknown_endtag(self, tag):        
        pass
   
    def unknown_charref(self,tag):
        pass

       
    def output(self):
        # Clean up text and return it
        content = "" . join(self.pieces)
        content = self._setEncoding(content)
        content = re.sub(r"[^0-9a-zA-ZæøåÆØÅ.?!\n\r]", " ", content)
        content = content.strip()
        #lines = content.replace("\t", " ").replace("\r","\n").splitlines()
        lines = content.splitlines()
        content = ""
        for line in lines:
            temp = line.strip()
            if temp == "" or temp == " ":
                lines.remove(line)
            else:    
                if len(temp.split(" ")) > 2:
                    content += line + "\n"
        return content
       
       
    def _setEncoding(self,text):
        try:
            text = unicode(text, "utf8").encode("utf8")
        except UnicodeError:
            try:
                text = unicode(text, "iso8859_1").encode("utf8")
            except UnicodeError:
                text = unicode(text, "ascii", "replace").encode("utf8")
        return text
   
    def demo(self):
        print 'This class takes an URL, and extracts the text it contains.'
        print 'It also removes special characters and numbers,'
        print 'and sentences must consist of at least'
        print '3 words to not be ignored.'
       
        print '\nFetching text from www.python.org'
        u = URLextracter()
        print "=" * 40
        print u.output()
    demo = classmethod(demo)

def demo():
    URLextracter.demo()

if __name__ == "__main__":
    demo()




import string, re, os

###
### Fallback syllable counter
###
### This is based on the algorithm in Greg Fast's perl module
### Lingua::EN::Syllable.
###

specialSyllables_en = """tottered 2
chummed 1
peeped 1
moustaches 2
shamefully 3
messieurs 2
satiated 4
sailmaker 4
sheered 1
disinterred 3
propitiatory 6
bepatched 2
particularized 5
caressed 2
trespassed 2
sepulchre 3
flapped 1
hemispheres 3
pencilled 2
motioned 2
poleman 2
slandered 2
sombre 2
etc 4
sidespring 2
mimes 1
effaces 2
mr 2
mrs 2
ms 1
dr 2
st 1
sr 2
jr 2
truckle 2
foamed 1
fringed 2
clattered 2
capered 2
mangroves 2
suavely 2
reclined 2
brutes 1
effaced 2
quivered 2
h'm 1
veriest 3
sententiously 4
deafened 2
manoeuvred 3
unstained 2
gaped 1
stammered 2
shivered 2
discoloured 3
gravesend 2
60 2
lb 1
unexpressed 3
greyish 2
unostentatious 5
"""

fallback_cache = {}

fallback_subsyl = ["cial", "tia", "cius", "cious", "gui", "ion", "iou",
                   "sia$", ".ely$"]

fallback_addsyl = ["ia", "riet", "dien", "iu", "io", "ii",
                   "[aeiouy]bl$", "mbl$",
                   "[aeiou]{3}",
                   "^mc", "ism$",
                   "(.)(?!\\1)([aeiouy])\\2l$",
                   "[^l]llien",
                   "^coad.", "^coag.", "^coal.", "^coax.",
                   "(.)(?!\\1)[gq]ua(.)(?!\\2)[aeiou]",
                   "dnt$"]


# Compile our regular expressions
for i in range(len(fallback_subsyl)):
    fallback_subsyl[i] = re.compile(fallback_subsyl[i])
for i in range(len(fallback_addsyl)):
    fallback_addsyl[i] = re.compile(fallback_addsyl[i])

def _normalize_word(word):
    return word.strip().lower()

# Read our syllable override file and stash that info in the cache
for line in specialSyllables_en.splitlines():
    line = line.strip()
    if line:
        toks = line.split()
        assert len(toks) == 2
        fallback_cache[_normalize_word(toks[0])] = int(toks[1])

def count(word):
    word = _normalize_word(word)

    if not word:
        return 0

    # Check for a cached syllable count
    count = fallback_cache.get(word, -1)
    if count > 0:
        return count

    # Remove final silent 'e'
    if word[-1] == "e":
        word = word[:-1]

    # Count vowel groups
    count = 0
    prev_was_vowel = 0
    for c in word:
        is_vowel = c in ("a", "e", "i", "o", "u", "y")
        if is_vowel and not prev_was_vowel:
            count += 1
        prev_was_vowel = is_vowel

    # Add & subtract syllables
    for r in fallback_addsyl:
        if r.search(word):
            count += 1
    for r in fallback_subsyl:
        if r.search(word):
            count -= 1

    # Cache the syllable count
    fallback_cache[word] = count

    return count

###
### Phoneme-driven syllable counting
###

def count_decomp(decomp):
    count = 0
    for unit in decomp:
        if gnoetics.phoneme.is_xstressed(unit):
            count += 1
    return count


from numpy import * 
from scipy import *
from numpy.random import randint
from numpy.linalg import norm
from matplotlib.pyplot import *
from matplotlib import rcParams
from xlrd import open_workbook

rcParams['text.usetex'] = True

'''
	Compare the similarity of tweets in a 26-dimensional space. Than ask if that angle is more than what
	could be expected from chance. Here, the random process must generate symbols with the same 
	distribution as the language in question. Otherwise, all tweets will be clustered together because
	they are all (hopefully) more like English than garbage. The assumption behind this is that tweets
	about a specific drug belong to a subcluster that has a unique frequency of symbols. There are
	better ways to project tweets onto a vector space, I'm sure. For now, though, let's try this
'''

#generate random letters
alphabet = list('abcdefghijklmnopqrstuvwxyz@?#')

def tweet_to_vector(tweet):
	answer = zeros((len(alphabet),))
	for position,letter in enumerate(alphabet):
		answer[position] = tweet.count(letter)
	return answer

def angle(first,second):
	return  dot(first,second)/(norm(first)*norm(second))

def similarity(tweets):
	return [[angle(tweet_to_vector(first),tweet_to_vector(second)) for first in tweets] for second in tweets]
	

#Load the coded data
def retrieve_tweets(drug_name):
	wb = open_workbook('../coding_master.xls')
	master_sheet = wb.sheet_by_index(0)
	return master_sheet.col_values(2)

#get actual_tweets				  
cocaine_tweets = retrieve_tweets('cocaine')
coke_similarity = similarity(cocaine_tweets)

tweet_length = 120
tweet_count = len(cocaine_tweets)
random_tweets = [''.join([alphabet[randint(len(alphabet))] for character in range(tweet_length)]) 
				  for tweet in range(tweet_count)]
	  
random_similarity = similarity(random_tweets) 
'''
figure()
imshow(random_similarity, interpolation='nearest', aspect='auto');colorbar()
title('Random Tweets')
xlabel('Tweet')
ylabel('Tweet')

figure()
imshow(coke_similarity, interpolation='nearest', aspect='auto');colorbar()
title('Cocaine Tweets')
xlabel('Tweet')
ylabel('Tweet')
'''

figure()
imshow(array(coke_similarity)-array(random_similarity), interpolation='nearest', aspect='auto');colorbar()
title('Difference in Similarity between pairs of cocaine tweets and random counterparts')
xlabel('Tweet')
ylabel('Tweet')

figure()
hist(tril(array(coke_similarity)-array(random_similarity)))
axvline(x=0,linewidth=3,color='r')
show()	


# -*- coding: utf-8 -*- 
# Sets the encoding to utf-8 to avoid problems with æøå

import glob
import re
import random
import math
import pickle
import os

from nltk.corpus import stopwords

from urlextracter import URLextracter
from sgmllib import *

class NaiveBayes(object):
       
    p_word_given_lang = {}
    
    training_files = []
    test_files = []
    candidate_languages = []

    def __init__(self):
        self.nor_stopwords = {}
        for word in stopwords.words('norwegian'):
            self.nor_stopwords[word] = True
            
        self.eng_stopwords = {}
        for word in stopwords.words('english'):
            self.eng_stopwords[word] = True
            
        self.load(os.path.join("files","lang_data.pickle"))

     
    """
    load pickled training results
    picklepath is the local path to your picklefile
    """        
    def load(self,picklepath):
        try:
            p = open(picklepath, 'rb')
            data = pickle.load(p)
            self.p_word_given_lang = data["p_word_given_lang"]
            self.candidate_languages = data["canidate_languages"]
            self.p_lang = data["p_lang"]
            self.vocabulary = data["vocabulary"]
        except IOError:
            self.p_word_given_lang = {}
            print "Nothing to load here!"
        
        
    """
    Train the classifier with data placed 
    in a folder named as the related language.
    Example: /path/to/files/eng/file01.txt
    """    
    def train(self, path):
        # Setup
        data_files = glob.glob(path + "/*/*")
        random.shuffle(data_files)
        
        self.training_files = data_files[0:300]
        self.test_files = data_files[300:]
        
        self.files = {}
        self.p_lang = {}
        
        # Calculate P(H)
        for file in self.training_files:
            values = file.split('/')
            lang = values[-2]
        
            if not self.p_lang.has_key(lang):
                self.p_lang[lang] = 0.0
            
            self.p_lang[lang] += 1.0
            
            if not self.files.has_key(lang):
                self.files[lang] = []
            
            f = open(file, 'r')
            self.files[lang] += f.read().replace("\n", " ").replace(".", "").split(" ")
            f.close()
            
        # Calculate probabilities
        for lang in self.p_lang.keys():
            self.p_lang[lang] /= len(self.training_files)
            
        self.vocabulary = self.__createVocabulary(self.files)
        
        # Calculate P(O | H) 
        p_word_given_lang = self.p_word_given_lang
        for lang in self.files.keys():
            p_word_given_lang[lang] = {}
            
            for word in self.vocabulary[lang].keys():
                p_word_given_lang[lang][word] = 1.0
            
            for word in self.files[lang]:
                if self.vocabulary[lang].has_key(word):
                    p_word_given_lang[lang][word] += 1.0
                    
            for word in self.vocabulary[lang].keys():
                p_word_given_lang[lang][word] /= len(self.files[lang]) + len(self.vocabulary[lang])
                
        print "Training finished...(training-set of size %d)" % len(self.training_files)
        self.p_word_given_lang = p_word_given_lang
        self.candidate_languages = self.files.keys()
        
        # Save result as a file
        output = open(os.path.join("files","lang_data.pickle"),'wb')
        data = {}
        data["p_word_given_lang"] = p_word_given_lang
        data["canidate_languages"] = self.files.keys()
        data["p_lang"] = self.p_lang
        data["vocabulary"] = self.vocabulary
        pickler = pickle.dump(data, output, -1)
        output.close()   
    
    """
    Filter out the words we're not interessted in
    and return a dictionary with all remaining words
    sorted by language.
    Example: vocabulary[eng] = {'lazy','fox',...}
    """
    def __createVocabulary(self, files):
        # Count number of occurance of each word
        word_count = {}
        for lang in files.keys():
            for word in files[lang]:
                if not word_count.has_key(word):
                    word_count[word] = 0
                word_count[word] += 1
        
        vocabulary = {}
        vocabulary['eng'] = {}
        vocabulary['no'] = {}
        for word in word_count.keys():
            if word_count[word] > 2:
                if word != '':
                    if not word in self.nor_stopwords:
                        vocabulary['no'][word] = True
                    if not word in self.eng_stopwords:
                        vocabulary['eng'][word] = True
        return vocabulary
    
    
    """
    Test the accuracy of the classifier.
    Provide test files as list or path.
    The path must be on the same form as when training.
    """
    def testAccuracy(self,test_files = ""):
        
        if test_files == "":
            print "No test files given"
            return
        elif os.path.isdir(str(test_files)):
            self.test_files = glob.glob(test_files + "/*/*")
            random.shuffle(self.test_files)
        else:
            self.test_files = test_files
  
        errors = 0.0
        total = 0.0
        
        # Use if test_files is provided as path
        #test_files = glob.glob(path + "/*/*")
        #random.shuffle(test_files)
        
        
        for file in self.test_files:
            values = file.split(os.sep)
            true_lang = values[-2]

            f = open(file, "r")    
            file_to_be_classified = f.read().replace("\n", " ").replace(".", "").split(" ")
            f.close()
            
            # Finds group with max P(O | H) * P(H)
            max_lang = 0
            max_p = 1
            for candidate_lang in self.candidate_languages:
                # Calculates P(O | H) * P(H) for candidate group
                p = math.log(self.p_lang[candidate_lang])
                for word in file_to_be_classified:
                    if self.vocabulary[candidate_lang].has_key(word):
                        p += math.log(self.p_word_given_lang[candidate_lang][word])
        
                if p > max_p or max_p == 1:
                    max_p = p
                    max_lang = candidate_lang
        
            total += 1.0
            if true_lang != max_lang:
                errors += 1.0
        print "Classifying finished...(test-set of size %d)" % len(self.test_files)
        print "Errors %d" % errors
        print "Total %d" % total
        print "Accuracy: %.3f" % (1.0 - errors/total)
    
    def classifyText(self, text):
        max_lang = 0
        max_p = 1
        for candidate_lang in self.candidate_languages:
            # Calculates P(O | H) * P(H) for candidate group
            p = math.log(self.p_lang[candidate_lang])
            words = text.split(' ')
            unknown_words = []
            known_words = []
            for word in words:
                if self.vocabulary[candidate_lang].has_key(word):
                    p += math.log(self.p_word_given_lang[candidate_lang][word])
                    if word not in known_words:
                        known_words.append(word)
                else:
                    if word not in unknown_words:
                        unknown_words.append(word)
            if p > max_p or max_p == 1:
                max_p = p
                max_lang = candidate_lang
        
        percent = (float(len(known_words)) / float(len(unknown_words)))
        # return unknown if the ratio of known words is less or equal to 0.25
        if percent <= 0.25:        
            max_lang = "unknown"
        
        return max_lang
    
    def classifyURL(self, url):
        ue = URLextracter(url)
        print 'Classifying %s' % url
        content = ue.output() 
        content = re.sub(r"[^a-zA-ZæøåÆØÅ]", " ", content)
        content = content.strip()
        return self.classifyText(content)
    
    def handle_decl(self,data):
        pass

    def report_unbalanced(self,tag):
        pass
    
    def demo(self):
        print "Demo of language classifier"
        print "=" * 40
        nb = NaiveBayes()
        nb.load(os.path.join("files","lang_data.pickle"))
        
        print "Classifying plain text(10 first sentences from \"nltk.corpus.abc.sents\")"
        print "=" * 40
        text = ""
        import nltk.corpus
        sents = nltk.corpus.abc.sents()
        for words in sents[0:10]:
            text+= " ".join(words) + "\n"
        print text
        print "=" * 40
        print "Languages is: %s" % nb.classifyText(text)
        
        print "\n"
        print "Classifying 10 URLs"
        print "=" * 40
        
        lang = nb.classifyURL("http://harvardscience.harvard.edu/")
        print "-->language: %s \n" % lang
        lang = nb.classifyURL("http://vg.no")
        print "-->language: %s \n" % lang
        lang = nb.classifyURL("http://bbc.co.uk")
        print "-->language: %s \n" % lang
        lang = nb.classifyURL("http://startsiden.no")
        print "-->language: %s \n" % lang
        lang = nb.classifyURL("http://news.com")
        print "-->language: %s \n" % lang
        lang = nb.classifyURL("http://www.munimadrid.es")
        print "-->language: %s \n" % lang
        lang = nb.classifyURL("http://www.welt.de/")
        print "-->language: %s \n" % lang
        lang = nb.classifyURL("http://www.news.pl/")
        print "-->language: %s \n" % lang
        lang = nb.classifyURL("http://www.ekstrabladet.dk/")
        print "-->language: %s \n" % lang
        lang = nb.classifyURL("http://www.gazzetta.it/")
        print "-->language: %s \n" % lang      
    demo = classmethod(demo)
    
def demo():
    NaiveBayes.demo()
    
if __name__=="__main__":
    NaiveBayes.demo()    


#!/usr/bin/env python

from distutils.core import setup

setup(name='toxtweet',
      version='0.10',
      description='ToxTweet- Tools for Toxicovigliance on Twitter',
      author='Michael Chary',
      author_email='mac389@gmail.com',
      url='http://www.python.org/sigs/distutils-sig/',
      py_modules=['better_twitter_search','classifying','crawler','FK','languageclassifier','lexdiv','loadTweets','main','medgraph','parser','readabilitytests','similarity','syllables_en','textanalyzer','twitterQuery','urlextracter','visualize_networks']
     )


import nltk.data
from nltk.tokenize import *
import syllables_en
from languageclassifier import *

class textanalyzer(object):

    tokenizer = RegexpTokenizer('(?u)\W+|\$[\d\.]+|\S+')
    special_chars = ['.', ',', '!', '?']
    lang = "eng"
    
    def __init__(self, lang):
        self.lang = lang
    
    def setLang(self,lang):
        self.lang = lang

    def analyzeText(self, text=''):
        words = self.getWords(text)
        charCount = self.getCharacterCount(words)
        wordCount = len(words)
        sentenceCount = len(self.getSentences(text))
        syllablesCount = self.countSyllables(words)
        complexwordsCount = self.countComplexWords(text)
        averageWordsPerSentence = wordCount/sentenceCount
        print ' Language: ' + self.lang
        print ' Number of characters: ' + str(charCount)
        print ' Number of words: ' + str(wordCount)
        print ' Number of sentences: ' + str(sentenceCount)
        print ' Number of syllables: ' + str(syllablesCount)
        print ' Number of complex words: ' + str(complexwordsCount)
        print ' Average words per sentence: ' + str(averageWordsPerSentence)
    #analyzeText = classmethod(analyzeText)  
        

    def getCharacterCount(self, words):
        characters = 0
        for word in words:
            word = self._setEncoding(word)
            characters += len(word.decode("utf-8"))
        return characters
    #getCharacterCount = classmethod(getCharacterCount)    
        
    def getWords(self, text=''):
        text = self._setEncoding(text)
        words = []
        words = self.tokenizer.tokenize(text)
        filtered_words = []
        for word in words:
            if word in self.special_chars or word == " ":
                pass
            else:
                new_word = word.replace(",","").replace(".","")
                new_word = new_word.replace("!","").replace("?","")
                filtered_words.append(new_word)
        return filtered_words
    #getWords = classmethod(getWords)
    
    def getSentences(self, text=''):
        sentences = []
        tokenizer = nltk.data.load('tokenizers/punkt/english.pickle')
        sentences = tokenizer.tokenize(text)
        return sentences
    #getSentences = classmethod(getSentences)
    
    def countSyllables(self, words = []):
#        if self.lang == "":
#            self.lang = NaiveBayes().classifyText(" " .join(words))
            
        if self.lang == "unknown":
            print "WARNING: Unknown language, using English\n"
            self.lang = "eng"    
        
        syllableCount = 0
        syllableCounter = {}
        syllableCounter['eng'] = syllables_en.count
        for word in words:
            syllableCount += syllableCounter[self.lang](word)
            
        return syllableCount
    #countSyllables = classmethod(countSyllables)
    
    
    #This method must be enhanced. At the moment it only
    #considers the number of syllables in a word.
    #This often results in that too many complex words are detected.
    def countComplexWords(self, text=''):
        words = self.getWords(text)
        sentences = len(self.getSentences(text));
        sentencesList = self.getSentences(text);
        complexWords = 0
        found = False;
        #Just for manual checking and debugging.
        cWords = []
        curWord = []
        
        for word in words:          
            curWord.append(word)
            if self.countSyllables(curWord)>= 3:
                
                #Checking proper nouns. If a word starts with a capital letter
                #and is NOT at the beginning of a sentence we don't add it
                #as a complex word.
                if not(word[0].isupper()):
                    complexWords += 1
                    #cWords.append(word)
                else:
                    for sentence in sentencesList:
                        if str(sentence).startswith(word):
                            found = True
                            break
                    
                    if found: 
                        complexWords+=1
                        found = False
                    
            curWord.remove(word)
        #print cWords
        return complexWords
    #countComplexWords = classmethod(countComplexWords)
    
    def _setEncoding(self,text):
        try:
            text = unicode(text, "utf8").encode("utf8")
        except UnicodeError:
            try:
                text = unicode(text, "iso8859_1").encode("utf8")
            except UnicodeError:
                text = unicode(text, "ascii", "replace").encode("utf8")
        return text
    #_setEncoding = classmethod(_setEncoding)
        
        
    def demo(self):
        text = "It is for us the living, rather,\
                \nto be dedicated here to the unfinished\
                \nwork which they who fought here have\
                \nthus far so nobly advanced. It is\
                \nrather for us to be here dedicated\
                \nto the great task remaining before us,\
                \nthat from these honored dead we take\
                \nincreased devotion to that cause for which they\
                \ngave the last full measure of devotion, that we\
                \nhere highly resolve that these dead shall not have\
                \ndied in vain, that this nation, under God, shall have a\
                \nnew birth of freedom, and that government of the people, by\
                \nthe people, for the people, shall not perish from this earth."
      
        print "The text : \n" + ("=" * 40)
        print text
        print ("=" * 40) + "\nHas the following statistics\n" + ("=" * 40)
        nb = NaiveBayes()
        ta = textanalyzer(nb.classifyText(text))
        ta.analyzeText(text)
        pass
    demo = classmethod(demo)
    
def demo():
    textanalyzer.demo()
    
if __name__ == "__main__":
    textanalyzer.demo()



from sys import argv

#Sanitize text
delchars = ''.join(c for c in map(chr, range(256)) if not c.isalnum())


def lexdiv(aList):
	return len(set(aList))/float(len(aList))
'''
if argv[1]:
	filename = argv[1]
	with open(filename,'r') as f:
		text = f.readlines()[0]
	cleaner = [word.translate(None, delchars).lower() for word in text.split()]
	print round(lexdiv(cleaner),3)
'''
def ld(filename):
	with open(filename,'r') as f:
		text = f.readlines()[0]
	cleaner = [word.translate(None, delchars).lower() for word in text.split()]
	return round(lexdiv(cleaner),3)


#Calculate FleshKincaid from Tweets

from nltk import * 
from simplejson import load

formulary = ['cocaine','marijuana','bath salts','heroin','opium','lsd','valium','ativan','ecstasy','ketamine']

tweets = []
#For FK any tweet will do
for drug in formulary:
	filename = "results_"+drug.replace(' ','_')+".txt"
	with open(filename,'r') as f:
		data = load(f)		
		tweets.append(getFK(data[0]['results'][0]['text']))
		
print tweets




from textanalyzer import *
import math

class ReadabilityTool:

    analyzedVars = {}
    text = ""
    lang = ""
    
    tests_given_lang = {}

    def __init__(self, text = ''):
        self.tests_given_lang['all'] = {}
        self.tests_given_lang['all']["ARI"] = self.ARI
        self.tests_given_lang['all']['Flesch Reading Ease'] = self.FleschReadingEase
        self.tests_given_lang['all']["Flesch-Kincaid Grade Level"] = self.FleschKincaidGradeLevel
        self.tests_given_lang['all']["Gunning Fog Index"] = self.GunningFogIndex
        self.tests_given_lang['all']["SMOG Index"] = self.SMOGIndex
        self.tests_given_lang['all']['Coleman Liau Index'] = self.ColemanLiauIndex
        self.tests_given_lang['all']['LIX'] = self.LIX
        self.tests_given_lang['all']['RIX'] = self.RIX
        
        self.tests_given_lang['eng'] = {}
        self.tests_given_lang['eng']["ARI"] = self.ARI
        self.tests_given_lang['eng']['Flesch Reading Ease'] = self.FleschReadingEase
        self.tests_given_lang['eng']["Flesch-Kincaid Grade Level"] = self.FleschKincaidGradeLevel
        self.tests_given_lang['eng']["Gunning Fog Index"] = self.GunningFogIndex
        self.tests_given_lang['eng']["SMOG Index"] = self.SMOGIndex
        self.tests_given_lang['eng']['Coleman Liau Index'] = self.ColemanLiauIndex
        self.tests_given_lang['eng']['LIX'] = self.LIX
        self.tests_given_lang['eng']['RIX'] = self.RIX
        
        self.tests_given_lang['no'] = {}
        self.tests_given_lang['no']["ARI"] = self.ARI
        self.tests_given_lang['no']['Coleman Liau Index'] = self.ColemanLiauIndex
        self.tests_given_lang['no']['LIX'] = self.LIX
        self.tests_given_lang['no']['RIX'] = self.RIX
        
        if text != '':
            self.__analyzeText(text)
                
    def __analyzeText(self, text=''):
        if text != '':
            if text != self.text:
                self.text = text
                t = textanalyzer('eng')
                t.analyzeText(text)
                words = t.getWords(text)
                charCount = t.getCharacterCount(words)
                wordCount = len(words)
                sentenceCount = len(t.getSentences(text))
                syllableCount = t.countSyllables(words)
                complexwordsCount = t.countComplexWords(text)
                averageWordsPerSentence = wordCount/sentenceCount
                
                analyzedVars = {}
                
                analyzedVars['words'] = words
                analyzedVars['charCount'] = float(charCount)
                analyzedVars['wordCount'] = float(wordCount)
                analyzedVars['sentenceCount'] = float(sentenceCount)
                analyzedVars['syllableCount'] = float(syllableCount)
                analyzedVars['complexwordCount'] = float(complexwordsCount)
                analyzedVars['averageWordsPerSentence'] = float(averageWordsPerSentence)
                self.analyzedVars = analyzedVars
        
     
    
    def ARI(self, text = ''):
        self.__analyzeText(text)
        score = 0.0
        analyzedVars = self.analyzedVars
        score = 4.71 * (analyzedVars['charCount'] / analyzedVars['wordCount']) + 0.5 * (analyzedVars['wordCount'] / analyzedVars['sentenceCount']) - 21.43
        return score
    
    def FleschReadingEase(self, text = ''):
        self.__analyzeText(text)
        score = 0.0
        analyzedVars = self.analyzedVars        
        score = 206.835 - (1.015 * (analyzedVars['averageWordsPerSentence'])) - (84.6 * (analyzedVars['syllableCount']/ analyzedVars['wordCount']))
        return score
    
    def FleschKincaidGradeLevel(self, text = ''):
        self.__analyzeText(text)
        score = 0.0
        analyzedVars = self.analyzedVars
        score = 0.39 * (analyzedVars['averageWordsPerSentence']) + 11.8 * (analyzedVars['syllableCount']/ analyzedVars['wordCount']) - 15.59
        return score
    
    def GunningFogIndex(self, text = ''):
        self.__analyzeText(text)
        score = 0.0
        analyzedVars = self.analyzedVars
        score = 0.4 * ((analyzedVars['averageWordsPerSentence']) + (100 * (analyzedVars['complexwordCount']/analyzedVars['wordCount'])))
        return score
    
    def SMOGIndex(self, text = ''):
        self.__analyzeText(text)
        score = 0.0
        analyzedVars = self.analyzedVars        
        score = (math.sqrt(analyzedVars['complexwordCount']*(30/analyzedVars['sentenceCount'])) + 3)
        return score
    
    def ColemanLiauIndex(self, text = ''):
        self.__analyzeText(text)
        score = 0.0
        analyzedVars = self.analyzedVars        
        score = (5.89*(analyzedVars['charCount']/analyzedVars['wordCount']))-(30*(analyzedVars['sentenceCount']/analyzedVars['wordCount']))-15.8
        return score
    
    def LIX(self, text = ''):
        self.__analyzeText(text)
        analyzedVars = self.analyzedVars
        score = 0.0
        longwords = 0.0
        for word in analyzedVars['words']:
            if len(word) >= 7:
                longwords += 1.0
        score = analyzedVars['wordCount'] / analyzedVars['sentenceCount'] + float(100 * longwords) / analyzedVars['wordCount']
        return score
    
    def RIX(self, text = ''):
        self.__analyzeText(text)
        analyzedVars = self.analyzedVars
        score = 0.0
        longwords = 0.0
        for word in analyzedVars['words']:
            if len(word) >= 7:
                longwords += 1.0
        score = longwords / analyzedVars['sentenceCount']
        return score
        
    
    def getReportAll(self, text = ''):
        self.__analyzeText(text)
#        ari = 0.0
#        fleschEase = 0.0
#        fleschGrade = 0.0
#        gunningFog = 0.0
#        smog = 0.0
#        coleman = 0.0
#        
#        ari = self.ARI()
#        fleschEase = self.FleschReadingEase()
#        fleschGrade = self.FleschKincaidGradeLevel()
#        gunningFog = self.GunningFogIndex()
#        smog = self.SMOGIndex()
#        coleman = self.ColemanLiauIndex()
#        lix = self.LIX()
#        rix = self.RIX()
#        
#        print '*' * 70
#        print ' ARI: %.1f' % ari
#        print ' Flesch Reading Ease: %.1f' % fleschEase
#        print ' FleschKincaid Grade Level: %.1f' % fleschGrade
#        print ' Gunning Fog: %.1f' % gunningFog
#        print ' SMOG Index: %.1f' % smog
#        print ' Coleman-Liau Index: %.1f' % coleman
#        print ' LIX : %.1f' % lix
#        print ' RIX : %.1f' % rix
#        print '*' * 70
        
        print "=" * 100
        print "Recommended tests for lang: %s" % self.lang 
        print "=" * 100
        for testname in self.tests_given_lang[self.lang].keys():
            print testname + " : %.2f" % self.tests_given_lang[self.lang][testname](text)
        print "=" * 100
        print "Other tests: (Warning! Use with care)"
        print "=" * 100 
        for testname in self.tests_given_lang["all"].keys():
            if not self.tests_given_lang[self.lang].has_key(testname):
                print testname + " : %.2f" % self.tests_given_lang["all"][testname](text) 
            
 
    def demo(self):
        self = ReadabilityTool()
        text = """
                It is for us the living, rather,
                to be dedicated here to the unfinished
                work which they who fought here have
                thus far so nobly advanced. It is
                rather for us to be here dedicated
                to the great task remaining before us,
                that from these honored dead we take 
                increased devotion to that cause for which they
                gave the last full measure of devotion, that we
                here highly resolve that these dead shall not have
                died in vain, that this nation, under God, shall have a
                new birth of freedom, and that government of the people, by
                the people, for the people, shall not perish from this earth.  
               """
       
        self.__analyzeText(text)
        self.getReportAll(text)
    demo = classmethod(demo) 
 
 
def demo():
    ReadabilityTool.demo()
    
if __name__ == "__main__":
    ReadabilityTool.demo()
